#ifndef _INIT_CPP
#define _INIT_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <CommCtrl.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"
#include "../../SharedClasses/CRC32/CRC.H"

#include "../../SharedSource/NSWFL.H"
#include "../../SharedSource/EventLog.H"

#include "Init.H"
#include "Entry.H"
#include "Routines.H"

#include "../CSockSrvr/CSockSrvr.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool InitializeApp(void)
{
    LoadString(ghAppInstance, IDS_APPCOPYRIGHT, gsAppCopyRight, sizeof(gsAppCopyRight));
    LoadString(ghAppInstance, IDS_FILEVERSION, gsFileVersion, sizeof(gsFileVersion));
    LoadString(ghAppInstance, IDS_TITLECAPTION, gsTitleCaption, sizeof(gsTitleCaption));
    LoadString(ghAppInstance, IDS_REGISTRYKEY, gsRegistryKey, sizeof(gsRegistryKey));

	memset(&gMem, 0, sizeof(gMem));

	if(!gMem.Init())
	{
        MessageBox(GetActiveWindow(), "Failed to Initialize the Memory Pool.", gsTitleCaption, MB_ICONERROR);
		return false;
	}

    INITCOMMONCONTROLSEX ICCEX;
    ICCEX.dwSize = sizeof(ICCEX);
    ICCEX.dwICC  = ICC_DATE_CLASSES;
    InitCommonControlsEx(&ICCEX);

	InitializeCRCTable();

	memset(&MDI, 0, sizeof(MDI));
	memset(&gServer, 0, sizeof(gServer));

	if(!InitializeCryptography(&gSC, CRYPTKEY, strlen(CRYPTKEY)))
	{
        MessageBox(GetActiveWindow(), "Failed to Initialize Cryptography Set CSockSrvr.", gsTitleCaption, MB_ICONERROR);
		RecordError("Failed to Initialize Cryptography Set CSockSrvr.");
        return false;
	}

	if(!GetRegistryValues())
	{
		MessageBox(GetActiveWindow(), "Failed to get registry values.", gsTitleCaption, 0);
		RecordError("Failed to get registry values.");
        return false;
	}

	RegisterEvents();

	if(!OpenLogFile())
	{
		MessageBox(GetActiveWindow(), "Failed to open log file.", gsTitleCaption, 0);
	}

	if(!gServer.Initialize(0, giMaxClients, !gbPacketMode, gbDropBadPackets, SENDBUFSZ, RECVBUFSZ))
    {
        WriteSysLogEx("Failed to Initialize the CSockSrvr.", EVENT_ERROR);
        return false;
    }

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool UninitializeApp(void)
{
    SaveRegistryValues();

	if(!gServer.UnInitialize())
    {
        WriteSysLogEx("Failed to UnInitialize the CSockSrvr", EVENT_ERROR);
        return 0;
    }

	DestroyCryptography(&gSC);

	CloseLogFile();

	if(!gMem.Destroy())
	{
        MessageBox(GetActiveWindow(), "Failed to Destroy the Memory Pool.", gsTitleCaption, MB_ICONERROR);
		RecordError("Failed to Initialize the CSockSrvr.");
		return false;
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
