#ifndef _Command_CPP
#define _Command_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <SQL.H>
#include <SqlExt.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "../../SharedSource/LZARICode.H"
#include "../../SharedSource/RLECompression.H"

#include "../../SharedClasses/CRC32/CRC.H"
#include "../../SharedClasses/SQLClass/cSQL.H"
#include "../../SharedClasses/SQLClass/cRecordSet.H"
#include "../../SharedClasses/CMemPool/CMemPool.H"
#include "../../SharedClasses/LZSS/LZSS_Compress.H"
#include "../../SharedClasses/LZSS/LZSS_Uncompress.H"

#include "../CSockSrvr/CSockSrvr.H"

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"

#include "Init.H"
#include "Entry.H"
#include "Routines.H"
#include "Command.H"
#include "SQLImport.H"
#include "Console.H"
#include "NewDB.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReceiveFileData(CSockSrvr *pSockSrvr, int iClient, char *sFileName);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	int ProcessCommand(CSockSrvr *pSockSrvr, int iClient, char *sRecvBuf, int iRecvBufSz);

	Processes a command from the peer.

	Possible return Values:
		CMD_OK    The command was a success, all is well.
		CMD_DONE  The command was a success, close the connection.
		CMD_ERROR The command failed due to an error.
*/
int ProcessCommand(CSockSrvr *pSockSrvr, int iClient, char *sCmdBuf, int iCmdBufSz)
{
    char sCmdData[SENDBUFSZ + RECVBUFSZ];

	int iSendBufSz = 0;
	int iCmdDataSz = 0;
	int iCmdFlagLength = 0;

    if((iCmdFlagLength = CmdCmp(sCmdBuf, "::SendingDBCreationSQL")))
    {
        sprintf(CCI[iClient].sFileName, "%s\\%s\\DBCreation.wpd", gsImportTempDir, CCI[iClient].sCompanyName);
        CreateFolderStructure(CCI[iClient].sFileName);

		if(ReceiveFileData(pSockSrvr, iClient, CCI[iClient].sFileName))
		{
			if(CreateNewDBFromDBCreationScript(&CCI[iClient].cCustSQL, pSockSrvr, iClient, CCI[iClient].sFileName))
			{
				pSockSrvr->SetNextSendData(iClient, "::BeginReplication->All");
				return CMD_OK;
			}
		}

		return CMD_ERROR;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::Complete")))
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "The process has been completed successfully.", EVENT_NONE);

		CCI[iClient].cIndexSQL.bThrowErrors = true;

		if(CCI[iClient].bRequestInit)
		{
			sprintf(sCmdData, "UPDATE [%s].[%s].[Companys]"
				" SET Init = 0"
				" WHERE CompName = '%s'",
				gsSQLIndexDatabase, gsDefaultDBO, CCI[iClient].sCompanyName);

			CCI[iClient].cIndexSQL.ExecuteNonQuery(sCmdData);
		}

		UpdateClientStats(pSockSrvr, iClient, &CCI[iClient].cIndexSQL);

		return CMD_DONE;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::Msg->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_INFO);
		return CMD_OK;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::DataCRC->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

		if(CCI[iClient].dwFileCRC != atol(sCmdData))
		{
			sprintf(sCmdData, "Received a corrupt file \"%s\". [%X-%X]",
				CCI[iClient].sFileName, CCI[iClient].dwFileCRC, atol(sCmdData));
			WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_ERROR);

			pSockSrvr->SetNextSendData(iClient, "::LastImportFailed");

			// FIXME: Should we return CMD_OK or CMD_ERROR?
			return CMD_OK;
		}

		bool bCheckExistance = true;

		if(CCI[iClient].bRequestInit)
		{
			bCheckExistance = false;
		}

		if(ImportSQLResults(pSockSrvr, iClient, bCheckExistance, CCI[iClient].sFileName))
        {
			pSockSrvr->SetNextSendData(iClient, "::LastImportSuccess");
			return CMD_OK;
        }
        else{
            // FIXME: Should we return CMD_OK or CMD_ERROR?
			pSockSrvr->SetNextSendData(iClient, "::LastImportFailed");
			return CMD_ERROR;
        }

		return CMD_OK;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::SendingFileData->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

        sprintf(CCI[iClient].sFileName, "%s\\%s\\%s.wpd", gsImportTempDir, CCI[iClient].sCompanyName, sCmdData);
        CreateFolderStructure(CCI[iClient].sFileName);

		if(ReceiveFileData(pSockSrvr, iClient, CCI[iClient].sFileName))
		{
			return CMD_OK;
		}

		return CMD_ERROR;
	}

	WriteLogEx(pSockSrvr->icClientID[iClient], "Received unknown command.", EVENT_ERROR);

	sCmdBuf[1024] = '\0';
	WriteLogEx(pSockSrvr->icClientID[iClient], sCmdBuf, EVENT_ERROR);
	return CMD_ERROR;	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool UpdateClientStats(CSockSrvr *pSockSrvr, int iClient, CSQL *lpSQL)
{
	SYSTEMTIME ST;

	char sSQL[2048];
	
	char lsDate[64];
	char lsTime[64];

	char sConnectTime[128];
	char sDisconnectTime[128];
	char sUserTime[128];
	char sKernelTime[128];

	GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &CCI[iClient].stConnectTime, NULL, lsDate, sizeof(lsDate));
	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT, &CCI[iClient].stConnectTime, NULL, lsTime, sizeof(lsTime));
	sprintf(sConnectTime, "%s %s", lsDate, lsTime);

	GetSystemTime(&ST);
	GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &ST, NULL, lsDate, sizeof(lsDate));
	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT, &ST, NULL, lsTime, sizeof(lsTime));
	sprintf(sDisconnectTime, "%s %s", lsDate, lsTime);

	SYSTEMTIME stCreationTime;
	SYSTEMTIME stExitTime;
	SYSTEMTIME stKernelTime;
	SYSTEMTIME stUserTime;

	FILETIME ftCreationTime;
	FILETIME ftExitTime;
	FILETIME ftKernelTime;
	FILETIME ftUserTime;

	memset(&stCreationTime, 0, sizeof(stCreationTime));
	memset(&stExitTime, 0, sizeof(stExitTime));
	memset(&stKernelTime, 0, sizeof(stKernelTime));
	memset(&stUserTime, 0, sizeof(stUserTime));

	memset(&ftCreationTime, 0, sizeof(ftCreationTime));
	memset(&ftExitTime, 0, sizeof(ftExitTime));
	memset(&ftKernelTime, 0, sizeof(ftKernelTime));
	memset(&ftUserTime, 0, sizeof(ftUserTime));

	if(GetThreadTimes( pSockSrvr->hcClient_Thread_Handle[iClient],
		&ftCreationTime, &ftExitTime, &ftKernelTime, &ftUserTime))
	{
		FileTimeToSystemTime(&ftCreationTime, &stCreationTime);
		FileTimeToSystemTime(&ftExitTime, &stExitTime);
		FileTimeToSystemTime(&ftKernelTime, &stKernelTime);
		FileTimeToSystemTime(&ftUserTime, &stUserTime);

		stKernelTime.wYear = 2005;
		stUserTime.wYear = 2005;

		GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &stUserTime, NULL, lsDate, sizeof(lsDate));
		GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT, &stUserTime, NULL, lsTime, sizeof(lsTime));
		sprintf(sUserTime, "%s %s", lsDate, lsTime);

		GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &stKernelTime, NULL, lsDate, sizeof(lsDate));
		GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT, &stKernelTime, NULL, lsTime, sizeof(lsTime));
		sprintf(sKernelTime, "%s %s", lsDate, lsTime);
	}
	else{
		strcpy(sUserTime, "");
		strcpy(sKernelTime, "");
	}

	sprintf(sSQL, "INSERT INTO [%s].[%s].[CompanyStats]"
		" (Company_Key, ConnectTime, DisconnectTime, BytesSent, BytesReceived, UserTime, KernelTime)"
		" select IsNull((select TOP 1 ID from Companys where CompName = '%s'), -1),"
		" '%s', '%s', %d, %d, '%s', '%s'",
		gsSQLIndexDatabase, gsDefaultDBO, CCI[iClient].sCompanyName,
		sConnectTime, sDisconnectTime, CCI[iClient].dwBytesSent, CCI[iClient].dwBytesRecv,
		sUserTime, sKernelTime);

	lpSQL->ExecuteNonQuery(sSQL);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReceiveFileData(CSockSrvr *pSockSrvr, int iClient, char *sFileName)
{
    FILE *hTargetHandle = NULL;

    char sText[1024];
	char sFullFile[MAX_PATH];
	char sRecvBuf[RECVBUFSZ + 1];
    int iRecvBufSz = 0;

	DWORD dwDataCRC = 0xffffffff;

	WriteLogEx(pSockSrvr->icClientID[iClient], "Begining file transfer.", EVENT_NONE);

	if(gbUseCompression)
	{
		sprintf(sFullFile, "%s.sec", sFileName);
	}
	else{
		strcpy(sFullFile, sFileName);
	}

	if( (hTargetHandle = fopen(sFullFile, "wb")) == NULL)
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to open export file for binary write.", EVENT_ERROR);
        return false;
    }

	if(gbDebugMode)
	{
		sprintf(sText, "File opened successfully. [%s]", sFullFile);
		WriteLogEx(pSockSrvr->icClientID[iClient], sText, EVENT_NONE);
	}

	while(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
    {
		if(pSockSrvr->GetClientData(iClient, sRecvBuf, &iRecvBufSz))
        {
			if(iRecvBufSz == 5)
			{
				sRecvBuf[iRecvBufSz] = '\0';
				if(strcmp(sRecvBuf, "::EOF") == 0)
				{
					break; // All is well.
				}
			}

			Decode(&CCI[iClient].SC, sRecvBuf, sRecvBuf, iRecvBufSz);
            dwDataCRC = PartialCRC(dwDataCRC, sRecvBuf, iRecvBufSz);
			fwrite(sRecvBuf, sizeof(char), iRecvBufSz, hTargetHandle);
		}
		else Sleep(1);
	}

	dwDataCRC = (dwDataCRC ^ 0xffffffff);
	CCI[iClient].dwFileCRC = dwDataCRC;

	sprintf(sRecvBuf, "Received data file. CRC: %X", dwDataCRC);
	WriteLogEx(pSockSrvr->icClientID[iClient], sRecvBuf, EVENT_NONE);

	fclose(hTargetHandle);

	if(gbUseCompression)
	{
		float dwBeforeSz = 0;
		float dwAfterSz = 0;

		sprintf(sFullFile, "%s.sec", sFileName);

		dwBeforeSz = (float) Get_FileSize(sFullFile);

		sprintf(sRecvBuf, "Decompressing file. (%.2f KB)", dwBeforeSz / 1024);
		WriteLogEx(pSockSrvr->icClientID[iClient], sRecvBuf, EVENT_NONE);

		if(strcmpi(gsCompressionMethod, "LZSS") == 0){
			LZSSDeCompressFile(sFullFile, sFileName);
		}
		else if(strcmpi(gsCompressionMethod, "LZARI") == 0){
			DecompressFile(sFullFile, sFileName);
		}
		else if(strcmpi(gsCompressionMethod, "RLE") == 0){
			RLEDecodeFile(sFullFile, sFileName);			
		}

		dwAfterSz = (float) Get_FileSize(sFileName);

		sprintf(sRecvBuf, "File size increased from %.2f KB to %.2f KB. (%.2f %% Decompression).",
			dwBeforeSz / 1024, dwAfterSz / 1024, (100 - ((dwBeforeSz / dwAfterSz) * 100)));
		WriteLogEx(pSockSrvr->icClientID[iClient], sRecvBuf, EVENT_NONE);

		if(gbDeleteCompressedFiles)
		{
			DeleteFile(sFullFile);
		}
	}

	if(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
	{
		return true;
	}

	WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to receive client data. Client disconncted.", EVENT_ERROR);

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
